# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["FeatureTransformHistoricalParams"]


class FeatureTransformHistoricalParams(TypedDict, total=False):
    transform: Required[str]
    """The type of transform requested.

    Currently supported are `xyavg`, `rebase`, `zscore`, `yoy`
    """

    agg: Optional[str]
    """
    If a `weekly` or `monthly` frequency is requested, this parameter allows to
    control how the returned data is aggregated over each period. `mean` and `last`
    are supported
    """

    end_date: Optional[str]
    """End of transformed feature data window (YYYY-MM-DD) - not relevant for yoy"""

    freq: Optional[str]
    """By default, time-series are returned using daily time frequency.

    Request resampled data using `weekly` or `monthly` as query parameter
    """

    number_of_years: int
    """Number of years to perform the average on. (valid for xyavg and yoy transforms)"""

    start_date: Optional[str]
    """Start of transformed feature data window (YYYY-MM-DD) - not relevant for yoy"""

    window: int
    """Number of observations used in the transformation window.

    (valid for xyavg and zscore transforms)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import List, Optional
from typing_extensions import Literal

from .asset import Asset
from ...tick import Tick
from ...._models import BaseModel

__all__ = ["IndicatorGetHedgeResponse", "Data", "DataContract", "DataContractIndicator", "DataContractIndicatorSery"]


class DataContractIndicatorSery(BaseModel):
    date: datetime.date

    value: float


class DataContractIndicator(BaseModel):
    name: str

    series: List[DataContractIndicatorSery]


class DataContract(BaseModel):
    asset_symbol: str

    indicator: DataContractIndicator

    name: str

    symbol: str

    last_tick: Optional[Tick] = None


class Data(BaseModel):
    asset: Asset

    contracts: List[DataContract]


class IndicatorGetHedgeResponse(BaseModel):
    data: Data

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .performance import (
    PerformanceResource,
    AsyncPerformanceResource,
    PerformanceResourceWithRawResponse,
    AsyncPerformanceResourceWithRawResponse,
    PerformanceResourceWithStreamingResponse,
    AsyncPerformanceResourceWithStreamingResponse,
)

__all__ = [
    "PerformanceResource",
    "AsyncPerformanceResource",
    "PerformanceResourceWithRawResponse",
    "AsyncPerformanceResourceWithRawResponse",
    "PerformanceResourceWithStreamingResponse",
    "AsyncPerformanceResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["ForexRetrieveParams"]


class ForexRetrieveParams(TypedDict, total=False):
    end_date: Optional[str]
    """End of forex data window (YYYY-MM-DD)"""

    foreign_per_usd: bool
    """Return prices as foreign currency per USD.

    If false, prices will be returned as USD per foreign currency.
    """

    start_date: Optional[str]
    """Start of forex data window (YYYY-MM-DD)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["ForecastGetLongTermParams"]


class ForecastGetLongTermParams(TypedDict, total=False):
    end_date: Optional[str]
    """End of forecast window (YYYY-MM-DD).

    The returned object will contain every forecast made between start*date and
    end_date. \\__Default value* : most recent date with forecasts
    """

    horizon: Optional[int]
    """Number of months to forecast.

    _Default value_ : Shortest available term for requested symbol.
    """

    rollover_type: Optional[str]
    """Which rollover method to use. _Default value_ : hist_vol"""

    start_date: Optional[str]
    """Start of forecast window (YYYY-MM-DD).

    The returned object will contain every forecast made between start*date and
    end_date. \\__Default value* : most recent date with forecasts
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["FeatureRetrieveHistoricalParams"]


class FeatureRetrieveHistoricalParams(TypedDict, total=False):
    add_strength_for_commodity: Optional[str]
    """
    If a future symbol is provided and a model for that commodity exists, a signed
    strength indicator will be returned in addition to the feature value
    """

    agg: Optional[str]
    """
    If a `weekly` or `monthly` frequency is requested, this parameter allows to
    control how the returned data is aggregated over each period. `mean` and `last`
    are supported
    """

    end_date: Optional[str]
    """End of feature data window (YYYY-MM-DD)"""

    freq: Optional[str]
    """By default, time-series are returned using daily time frequency.

    Request resampled data using `weekly` or `monthly` as query parameter
    """

    start_date: Optional[str]
    """Start of feature data window (YYYY-MM-DD)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import List, Optional
from typing_extensions import Literal

from .asset import Asset
from ...._models import BaseModel
from ...feature_category import FeatureCategory

__all__ = ["ForecastGetLongTermResponse", "Data", "DataLongTermForecast", "DataMarketDriver"]


class DataLongTermForecast(BaseModel):
    contract: str

    date: datetime.date

    price_increase: bool


class DataMarketDriver(BaseModel):
    categories: List[FeatureCategory]

    forecast_date: datetime.datetime

    horizon: int

    model: str

    target_date_contract: str


class Data(BaseModel):
    asset: Asset

    long_term_forecast: List[DataLongTermForecast]

    market_drivers: Optional[List[DataMarketDriver]] = None


class ForecastGetLongTermResponse(BaseModel):
    data: Data

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["FutureGetTradingCalendarParams"]


class FutureGetTradingCalendarParams(TypedDict, total=False):
    end_date: Optional[str]
    """End of trading calendar window (YYYY-MM-DD)"""

    start_date: Optional[str]
    """Start of trading calendar window (YYYY-MM-DD)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .performance import (
    PerformanceResource,
    AsyncPerformanceResource,
    PerformanceResourceWithRawResponse,
    AsyncPerformanceResourceWithRawResponse,
    PerformanceResourceWithStreamingResponse,
    AsyncPerformanceResourceWithStreamingResponse,
)

__all__ = ["ModelsResource", "AsyncModelsResource"]


class ModelsResource(SyncAPIResource):
    @cached_property
    def performance(self) -> PerformanceResource:
        return PerformanceResource(self._client)

    @cached_property
    def with_raw_response(self) -> ModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return ModelsResourceWithStreamingResponse(self)


class AsyncModelsResource(AsyncAPIResource):
    @cached_property
    def performance(self) -> AsyncPerformanceResource:
        return AsyncPerformanceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncModelsResourceWithStreamingResponse(self)


class ModelsResourceWithRawResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

    @cached_property
    def performance(self) -> PerformanceResourceWithRawResponse:
        return PerformanceResourceWithRawResponse(self._models.performance)


class AsyncModelsResourceWithRawResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

    @cached_property
    def performance(self) -> AsyncPerformanceResourceWithRawResponse:
        return AsyncPerformanceResourceWithRawResponse(self._models.performance)


class ModelsResourceWithStreamingResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

    @cached_property
    def performance(self) -> PerformanceResourceWithStreamingResponse:
        return PerformanceResourceWithStreamingResponse(self._models.performance)


class AsyncModelsResourceWithStreamingResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

    @cached_property
    def performance(self) -> AsyncPerformanceResourceWithStreamingResponse:
        return AsyncPerformanceResourceWithStreamingResponse(self._models.performance)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AssetListResponse"]


class AssetListResponse(BaseModel):
    data: List[str]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .feature_category import FeatureCategory

__all__ = ["SupplyRetrieveResponse", "Data"]


class Data(BaseModel):
    commodity: str

    country: str

    country_code: str

    date: datetime

    feature_contributions: Optional[List[FeatureCategory]] = None

    forecasted_supply: float

    model: str

    reported_supply: float

    reporting_agency: str

    symbol: str

    unit: str


class SupplyRetrieveResponse(BaseModel):
    data: List[Data]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .data import (
    DataResource,
    AsyncDataResource,
    DataResourceWithRawResponse,
    AsyncDataResourceWithRawResponse,
    DataResourceWithStreamingResponse,
    AsyncDataResourceWithStreamingResponse,
)
from .ping import (
    PingResource,
    AsyncPingResource,
    PingResourceWithRawResponse,
    AsyncPingResourceWithRawResponse,
    PingResourceWithStreamingResponse,
    AsyncPingResourceWithStreamingResponse,
)
from .forex import (
    ForexResource,
    AsyncForexResource,
    ForexResourceWithRawResponse,
    AsyncForexResourceWithRawResponse,
    ForexResourceWithStreamingResponse,
    AsyncForexResourceWithStreamingResponse,
)
from .assets import (
    AssetsResource,
    AsyncAssetsResource,
    AssetsResourceWithRawResponse,
    AsyncAssetsResourceWithRawResponse,
    AssetsResourceWithStreamingResponse,
    AsyncAssetsResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .supply import (
    SupplyResource,
    AsyncSupplyResource,
    SupplyResourceWithRawResponse,
    AsyncSupplyResourceWithRawResponse,
    SupplyResourceWithStreamingResponse,
    AsyncSupplyResourceWithStreamingResponse,
)
from .features import (
    FeaturesResource,
    AsyncFeaturesResource,
    FeaturesResourceWithRawResponse,
    AsyncFeaturesResourceWithRawResponse,
    FeaturesResourceWithStreamingResponse,
    AsyncFeaturesResourceWithStreamingResponse,
)

__all__ = [
    "PingResource",
    "AsyncPingResource",
    "PingResourceWithRawResponse",
    "AsyncPingResourceWithRawResponse",
    "PingResourceWithStreamingResponse",
    "AsyncPingResourceWithStreamingResponse",
    "DataResource",
    "AsyncDataResource",
    "DataResourceWithRawResponse",
    "AsyncDataResourceWithRawResponse",
    "DataResourceWithStreamingResponse",
    "AsyncDataResourceWithStreamingResponse",
    "AssetsResource",
    "AsyncAssetsResource",
    "AssetsResourceWithRawResponse",
    "AsyncAssetsResourceWithRawResponse",
    "AssetsResourceWithStreamingResponse",
    "AsyncAssetsResourceWithStreamingResponse",
    "ForexResource",
    "AsyncForexResource",
    "ForexResourceWithRawResponse",
    "AsyncForexResourceWithRawResponse",
    "ForexResourceWithStreamingResponse",
    "AsyncForexResourceWithStreamingResponse",
    "FeaturesResource",
    "AsyncFeaturesResource",
    "FeaturesResourceWithRawResponse",
    "AsyncFeaturesResourceWithRawResponse",
    "FeaturesResourceWithStreamingResponse",
    "AsyncFeaturesResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "SupplyResource",
    "AsyncSupplyResource",
    "SupplyResourceWithRawResponse",
    "AsyncSupplyResourceWithRawResponse",
    "SupplyResourceWithStreamingResponse",
    "AsyncSupplyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime

from .._models import BaseModel

__all__ = ["ForexData", "Price"]


class Price(BaseModel):
    date: datetime

    price: float


class ForexData(BaseModel):
    code: str

    market_date: datetime

    name: str

    prices: List[Price]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.models import performance_retrieve_params
from ...types.models.performance_list_response import PerformanceListResponse
from ...types.models.performance_retrieve_response import PerformanceRetrieveResponse

__all__ = ["PerformanceResource", "AsyncPerformanceResource"]


class PerformanceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PerformanceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PerformanceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PerformanceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return PerformanceResourceWithStreamingResponse(self)

    def retrieve(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        horizon: Optional[int] | NotGiven = NOT_GIVEN,
        metric: Optional[str] | NotGiven = NOT_GIVEN,
        sigma: float | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        threshold_on_actual: Optional[float] | NotGiven = NOT_GIVEN,
        threshold_on_forecast: Optional[float] | NotGiven = NOT_GIVEN,
        use_run_date: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PerformanceRetrieveResponse:
        """
        The metrics values are not provided as-is, but the relevant data and recommended
        aggregation method for each metric is returned.

        Args:
          symbol: Future symbol

          end_date: End of the performance assessment window (YYYY-MM-DD)

          horizon: the horizon (number of business days in the future) of the model of interest.
              Default, returns for all horizons

          metric: the requested metric as described in the metadata. Default: hitrate

          sigma: Number of standard deviations to calculate for ACE. Unused for other metrics.

          start_date: Start of the performance assessment window (YYYY-MM-DD)

          threshold_on_actual: Allows to assess directional performance filtered on days where the % realized
              price change is greater than threshold. Default: no filter. Valid only for
              hitrate style metrics. Eg. enter 0.02 for 2%

          threshold_on_forecast: Allows to assess directional performance filtered on days where the % expected
              change of the forecast is greater than threshold. Default: no filter. Valid only
              for hitrate style metrics. Eg. enter 0.02 for 2%

          use_run_date: Use the run date of the forecast to filter the performance assessment window. If
              false, use the target date. Setting this to false makes the result deterministic
              from day to day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return self._get(
            f"/v1/models/performance/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "horizon": horizon,
                        "metric": metric,
                        "sigma": sigma,
                        "start_date": start_date,
                        "threshold_on_actual": threshold_on_actual,
                        "threshold_on_forecast": threshold_on_forecast,
                        "use_run_date": use_run_date,
                    },
                    performance_retrieve_params.PerformanceRetrieveParams,
                ),
            ),
            cast_to=PerformanceRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PerformanceListResponse:
        """
        Returns the list of success data that can be requested to assess our forecasting
        models performance
        """
        return self._get(
            "/v1/models/performance",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PerformanceListResponse,
        )


class AsyncPerformanceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPerformanceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPerformanceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPerformanceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncPerformanceResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        horizon: Optional[int] | NotGiven = NOT_GIVEN,
        metric: Optional[str] | NotGiven = NOT_GIVEN,
        sigma: float | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        threshold_on_actual: Optional[float] | NotGiven = NOT_GIVEN,
        threshold_on_forecast: Optional[float] | NotGiven = NOT_GIVEN,
        use_run_date: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PerformanceRetrieveResponse:
        """
        The metrics values are not provided as-is, but the relevant data and recommended
        aggregation method for each metric is returned.

        Args:
          symbol: Future symbol

          end_date: End of the performance assessment window (YYYY-MM-DD)

          horizon: the horizon (number of business days in the future) of the model of interest.
              Default, returns for all horizons

          metric: the requested metric as described in the metadata. Default: hitrate

          sigma: Number of standard deviations to calculate for ACE. Unused for other metrics.

          start_date: Start of the performance assessment window (YYYY-MM-DD)

          threshold_on_actual: Allows to assess directional performance filtered on days where the % realized
              price change is greater than threshold. Default: no filter. Valid only for
              hitrate style metrics. Eg. enter 0.02 for 2%

          threshold_on_forecast: Allows to assess directional performance filtered on days where the % expected
              change of the forecast is greater than threshold. Default: no filter. Valid only
              for hitrate style metrics. Eg. enter 0.02 for 2%

          use_run_date: Use the run date of the forecast to filter the performance assessment window. If
              false, use the target date. Setting this to false makes the result deterministic
              from day to day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return await self._get(
            f"/v1/models/performance/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_date": end_date,
                        "horizon": horizon,
                        "metric": metric,
                        "sigma": sigma,
                        "start_date": start_date,
                        "threshold_on_actual": threshold_on_actual,
                        "threshold_on_forecast": threshold_on_forecast,
                        "use_run_date": use_run_date,
                    },
                    performance_retrieve_params.PerformanceRetrieveParams,
                ),
            ),
            cast_to=PerformanceRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PerformanceListResponse:
        """
        Returns the list of success data that can be requested to assess our forecasting
        models performance
        """
        return await self._get(
            "/v1/models/performance",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PerformanceListResponse,
        )


class PerformanceResourceWithRawResponse:
    def __init__(self, performance: PerformanceResource) -> None:
        self._performance = performance

        self.retrieve = to_raw_response_wrapper(
            performance.retrieve,
        )
        self.list = to_raw_response_wrapper(
            performance.list,
        )


class AsyncPerformanceResourceWithRawResponse:
    def __init__(self, performance: AsyncPerformanceResource) -> None:
        self._performance = performance

        self.retrieve = async_to_raw_response_wrapper(
            performance.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            performance.list,
        )


class PerformanceResourceWithStreamingResponse:
    def __init__(self, performance: PerformanceResource) -> None:
        self._performance = performance

        self.retrieve = to_streamed_response_wrapper(
            performance.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            performance.list,
        )


class AsyncPerformanceResourceWithStreamingResponse:
    def __init__(self, performance: AsyncPerformanceResource) -> None:
        self._performance = performance

        self.retrieve = async_to_streamed_response_wrapper(
            performance.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            performance.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hedgewise import Hedgewise, AsyncHedgewise
from tests.utils import assert_matches_type
from hedgewise.types import (
    TransformedFeature,
    FeatureListResponse,
    FeatureRetrieveHistoricalResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFeatures:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hedgewise) -> None:
        feature = client.features.list()
        assert_matches_type(FeatureListResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: Hedgewise) -> None:
        feature = client.features.list(
            dataset_key="technical_macro_v1_2025",
            symbol="ZC",
        )
        assert_matches_type(FeatureListResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hedgewise) -> None:
        response = client.features.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = response.parse()
        assert_matches_type(FeatureListResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hedgewise) -> None:
        with client.features.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = response.parse()
            assert_matches_type(FeatureListResponse, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_weighted_index(self, client: Hedgewise) -> None:
        feature = client.features.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_weighted_index_with_all_params(self, client: Hedgewise) -> None:
        feature = client.features.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
            end_date="2025-04-25",
            start_date="2025-03-24",
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_weighted_index(self, client: Hedgewise) -> None:
        response = client.features.with_raw_response.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = response.parse()
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_weighted_index(self, client: Hedgewise) -> None:
        with client.features.with_streaming_response.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = response.parse()
            assert_matches_type(TransformedFeature, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_historical(self, client: Hedgewise) -> None:
        feature = client.features.retrieve_historical(
            feature_code="vietnam_t2mean",
        )
        assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_historical_with_all_params(self, client: Hedgewise) -> None:
        feature = client.features.retrieve_historical(
            feature_code="vietnam_t2mean",
            add_strength_for_commodity="KC",
            agg="mean",
            end_date="2025-04-25",
            freq="weekly",
            start_date="2025-03-24",
        )
        assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_historical(self, client: Hedgewise) -> None:
        response = client.features.with_raw_response.retrieve_historical(
            feature_code="vietnam_t2mean",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = response.parse()
        assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_historical(self, client: Hedgewise) -> None:
        with client.features.with_streaming_response.retrieve_historical(
            feature_code="vietnam_t2mean",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = response.parse()
            assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve_historical(self, client: Hedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `feature_code` but received ''"):
            client.features.with_raw_response.retrieve_historical(
                feature_code="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_transform_historical(self, client: Hedgewise) -> None:
        feature = client.features.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_transform_historical_with_all_params(self, client: Hedgewise) -> None:
        feature = client.features.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
            agg="mean",
            end_date="2025-04-25",
            freq="weekly",
            number_of_years=5,
            start_date="2025-03-24",
            window=20,
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_transform_historical(self, client: Hedgewise) -> None:
        response = client.features.with_raw_response.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = response.parse()
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_transform_historical(self, client: Hedgewise) -> None:
        with client.features.with_streaming_response.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = response.parse()
            assert_matches_type(TransformedFeature, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_transform_historical(self, client: Hedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `feature_code` but received ''"):
            client.features.with_raw_response.transform_historical(
                feature_code="",
                transform="xyavg",
            )


class TestAsyncFeatures:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.list()
        assert_matches_type(FeatureListResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.list(
            dataset_key="technical_macro_v1_2025",
            symbol="ZC",
        )
        assert_matches_type(FeatureListResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.features.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = await response.parse()
        assert_matches_type(FeatureListResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHedgewise) -> None:
        async with async_client.features.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = await response.parse()
            assert_matches_type(FeatureListResponse, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_weighted_index(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_weighted_index_with_all_params(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
            end_date="2025-04-25",
            start_date="2025-03-24",
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_weighted_index(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.features.with_raw_response.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = await response.parse()
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_weighted_index(self, async_client: AsyncHedgewise) -> None:
        async with async_client.features.with_streaming_response.get_weighted_index(
            feature_codes=["string"],
            index_label="user_defined_index",
            weights=[0],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = await response.parse()
            assert_matches_type(TransformedFeature, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_historical(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.retrieve_historical(
            feature_code="vietnam_t2mean",
        )
        assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_historical_with_all_params(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.retrieve_historical(
            feature_code="vietnam_t2mean",
            add_strength_for_commodity="KC",
            agg="mean",
            end_date="2025-04-25",
            freq="weekly",
            start_date="2025-03-24",
        )
        assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_historical(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.features.with_raw_response.retrieve_historical(
            feature_code="vietnam_t2mean",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = await response.parse()
        assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_historical(self, async_client: AsyncHedgewise) -> None:
        async with async_client.features.with_streaming_response.retrieve_historical(
            feature_code="vietnam_t2mean",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = await response.parse()
            assert_matches_type(FeatureRetrieveHistoricalResponse, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve_historical(self, async_client: AsyncHedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `feature_code` but received ''"):
            await async_client.features.with_raw_response.retrieve_historical(
                feature_code="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_transform_historical(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_transform_historical_with_all_params(self, async_client: AsyncHedgewise) -> None:
        feature = await async_client.features.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
            agg="mean",
            end_date="2025-04-25",
            freq="weekly",
            number_of_years=5,
            start_date="2025-03-24",
            window=20,
        )
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_transform_historical(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.features.with_raw_response.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        feature = await response.parse()
        assert_matches_type(TransformedFeature, feature, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_transform_historical(self, async_client: AsyncHedgewise) -> None:
        async with async_client.features.with_streaming_response.transform_historical(
            feature_code="vietnam_t2mean",
            transform="xyavg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            feature = await response.parse()
            assert_matches_type(TransformedFeature, feature, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_transform_historical(self, async_client: AsyncHedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `feature_code` but received ''"):
            await async_client.features.with_raw_response.transform_historical(
                feature_code="",
                transform="xyavg",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tick import Tick as Tick
from .forex_data import ForexData as ForexData
from .feature_category import FeatureCategory as FeatureCategory
from .asset_list_response import AssetListResponse as AssetListResponse
from .feature_list_params import FeatureListParams as FeatureListParams
from .forex_list_response import ForexListResponse as ForexListResponse
from .transformed_feature import TransformedFeature as TransformedFeature
from .supply_list_response import SupplyListResponse as SupplyListResponse
from .feature_list_response import FeatureListResponse as FeatureListResponse
from .forex_retrieve_params import ForexRetrieveParams as ForexRetrieveParams
from .data_retrieve_response import DataRetrieveResponse as DataRetrieveResponse
from .supply_retrieve_params import SupplyRetrieveParams as SupplyRetrieveParams
from .forex_retrieve_response import ForexRetrieveResponse as ForexRetrieveResponse
from .supply_retrieve_response import SupplyRetrieveResponse as SupplyRetrieveResponse
from .feature_get_weighted_index_params import FeatureGetWeightedIndexParams as FeatureGetWeightedIndexParams
from .feature_retrieve_historical_params import FeatureRetrieveHistoricalParams as FeatureRetrieveHistoricalParams
from .feature_transform_historical_params import FeatureTransformHistoricalParams as FeatureTransformHistoricalParams
from .feature_retrieve_historical_response import FeatureRetrieveHistoricalResponse as FeatureRetrieveHistoricalResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["PerformanceRetrieveParams"]


class PerformanceRetrieveParams(TypedDict, total=False):
    end_date: Optional[str]
    """End of the performance assessment window (YYYY-MM-DD)"""

    horizon: Optional[int]
    """the horizon (number of business days in the future) of the model of interest.

    Default, returns for all horizons
    """

    metric: Optional[str]
    """the requested metric as described in the metadata. Default: hitrate"""

    sigma: float
    """Number of standard deviations to calculate for ACE. Unused for other metrics."""

    start_date: Optional[str]
    """Start of the performance assessment window (YYYY-MM-DD)"""

    threshold_on_actual: Optional[float]
    """
    Allows to assess directional performance filtered on days where the % realized
    price change is greater than threshold. Default: no filter. Valid only for
    hitrate style metrics. Eg. enter 0.02 for 2%
    """

    threshold_on_forecast: Optional[float]
    """
    Allows to assess directional performance filtered on days where the % expected
    change of the forecast is greater than threshold. Default: no filter. Valid only
    for hitrate style metrics. Eg. enter 0.02 for 2%
    """

    use_run_date: bool
    """Use the run date of the forecast to filter the performance assessment window.

    If false, use the target date. Setting this to false makes the result
    deterministic from day to day.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hedgewise import Hedgewise, AsyncHedgewise
from tests.utils import assert_matches_type
from hedgewise.types.assets import (
    FutureListResponse,
    FutureGetTradingCalendarResponse,
    FutureGetHistoricalPricesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFutures:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hedgewise) -> None:
        future = client.assets.futures.list()
        assert_matches_type(FutureListResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hedgewise) -> None:
        response = client.assets.futures.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        future = response.parse()
        assert_matches_type(FutureListResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hedgewise) -> None:
        with client.assets.futures.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            future = response.parse()
            assert_matches_type(FutureListResponse, future, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_historical_prices(self, client: Hedgewise) -> None:
        future = client.assets.futures.get_historical_prices(
            symbol="ZC",
        )
        assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_historical_prices_with_all_params(self, client: Hedgewise) -> None:
        future = client.assets.futures.get_historical_prices(
            symbol="ZC",
            active_contracts_only=True,
            back_adjust=True,
            contract="2025H",
            end_date="2025-04-24",
            rollover_method="hist_vol",
            start_date="2025-04-24",
        )
        assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_historical_prices(self, client: Hedgewise) -> None:
        response = client.assets.futures.with_raw_response.get_historical_prices(
            symbol="ZC",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        future = response.parse()
        assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_historical_prices(self, client: Hedgewise) -> None:
        with client.assets.futures.with_streaming_response.get_historical_prices(
            symbol="ZC",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            future = response.parse()
            assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_historical_prices(self, client: Hedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `symbol` but received ''"):
            client.assets.futures.with_raw_response.get_historical_prices(
                symbol="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_get_trading_calendar(self, client: Hedgewise) -> None:
        future = client.assets.futures.get_trading_calendar(
            symbol="ZC",
        )
        assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_trading_calendar_with_all_params(self, client: Hedgewise) -> None:
        future = client.assets.futures.get_trading_calendar(
            symbol="ZC",
            end_date="2025-04-25",
            start_date="2025-04-17",
        )
        assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_trading_calendar(self, client: Hedgewise) -> None:
        response = client.assets.futures.with_raw_response.get_trading_calendar(
            symbol="ZC",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        future = response.parse()
        assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_trading_calendar(self, client: Hedgewise) -> None:
        with client.assets.futures.with_streaming_response.get_trading_calendar(
            symbol="ZC",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            future = response.parse()
            assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_trading_calendar(self, client: Hedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `symbol` but received ''"):
            client.assets.futures.with_raw_response.get_trading_calendar(
                symbol="",
            )


class TestAsyncFutures:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHedgewise) -> None:
        future = await async_client.assets.futures.list()
        assert_matches_type(FutureListResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.assets.futures.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        future = await response.parse()
        assert_matches_type(FutureListResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHedgewise) -> None:
        async with async_client.assets.futures.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            future = await response.parse()
            assert_matches_type(FutureListResponse, future, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_historical_prices(self, async_client: AsyncHedgewise) -> None:
        future = await async_client.assets.futures.get_historical_prices(
            symbol="ZC",
        )
        assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_historical_prices_with_all_params(self, async_client: AsyncHedgewise) -> None:
        future = await async_client.assets.futures.get_historical_prices(
            symbol="ZC",
            active_contracts_only=True,
            back_adjust=True,
            contract="2025H",
            end_date="2025-04-24",
            rollover_method="hist_vol",
            start_date="2025-04-24",
        )
        assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_historical_prices(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.assets.futures.with_raw_response.get_historical_prices(
            symbol="ZC",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        future = await response.parse()
        assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_historical_prices(self, async_client: AsyncHedgewise) -> None:
        async with async_client.assets.futures.with_streaming_response.get_historical_prices(
            symbol="ZC",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            future = await response.parse()
            assert_matches_type(FutureGetHistoricalPricesResponse, future, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_historical_prices(self, async_client: AsyncHedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `symbol` but received ''"):
            await async_client.assets.futures.with_raw_response.get_historical_prices(
                symbol="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_trading_calendar(self, async_client: AsyncHedgewise) -> None:
        future = await async_client.assets.futures.get_trading_calendar(
            symbol="ZC",
        )
        assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_trading_calendar_with_all_params(self, async_client: AsyncHedgewise) -> None:
        future = await async_client.assets.futures.get_trading_calendar(
            symbol="ZC",
            end_date="2025-04-25",
            start_date="2025-04-17",
        )
        assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_trading_calendar(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.assets.futures.with_raw_response.get_trading_calendar(
            symbol="ZC",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        future = await response.parse()
        assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_trading_calendar(self, async_client: AsyncHedgewise) -> None:
        async with async_client.assets.futures.with_streaming_response.get_trading_calendar(
            symbol="ZC",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            future = await response.parse()
            assert_matches_type(FutureGetTradingCalendarResponse, future, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_trading_calendar(self, async_client: AsyncHedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `symbol` but received ''"):
            await async_client.assets.futures.with_raw_response.get_trading_calendar(
                symbol="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["ForecastGetParams"]


class ForecastGetParams(TypedDict, total=False):
    end_date: Optional[str]
    """End of forecast window (YYYY-MM-DD).

    The returned object will contain every forecast made between start*date and
    end_date. \\__Default value* : most recent date with forecasts
    """

    estimate_uncertainty: bool
    """Estimate prediction uncertainty based on recent historical accuracy."""

    get_market_drivers: bool
    """Return market drivers for each forecast."""

    get_moving_averages: bool
    """Return moving averages for each forecast."""

    interpolate: bool
    """Interpolate between forecast horizons and return daily forecast."""

    model_name: Optional[str]
    """
    Select a specific data model to use when generating a forecast for a future
    symbol.
    """

    price_collar_sigma: float
    """Apply an empirical price collar to the forecasts.

    This regulates the forecast when it suggests implausibly large price changes. A
    smaller number results in a more aggressive collar. Must be a positive number.
    """

    start_date: Optional[str]
    """Start of forecast window (YYYY-MM-DD).

    The returned object will contain every forecast made between start*date and
    end_date. \\__Default value* : most recent date with forecasts
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["FeatureRetrieveHistoricalResponse", "Data"]


class Data(BaseModel):
    date: datetime.date

    feature_code: str

    value: float

    phenology_stage: Optional[str] = None

    strength: Optional[float] = None


class FeatureRetrieveHistoricalResponse(BaseModel):
    data: List[Data]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import forex_retrieve_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.forex_list_response import ForexListResponse
from ..types.forex_retrieve_response import ForexRetrieveResponse

__all__ = ["ForexResource", "AsyncForexResource"]


class ForexResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ForexResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ForexResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ForexResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return ForexResourceWithStreamingResponse(self)

    def retrieve(
        self,
        code: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        foreign_per_usd: bool | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForexRetrieveResponse:
        """
        Returns forex price history for a given currency code.

        Args:
          code: Forex code

          end_date: End of forex data window (YYYY-MM-DD)

          foreign_per_usd: Return prices as foreign currency per USD. If false, prices will be returned as
              USD per foreign currency.

          start_date: Start of forex data window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not code:
            raise ValueError(f"Expected a non-empty value for `code` but received {code!r}")
        return self._get(
            f"/v1/forex/{code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "foreign_per_usd": foreign_per_usd,
                        "start_date": start_date,
                    },
                    forex_retrieve_params.ForexRetrieveParams,
                ),
            ),
            cast_to=ForexRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForexListResponse:
        """Get most recent forex prices for all currencies"""
        return self._get(
            "/v1/forex",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForexListResponse,
        )


class AsyncForexResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncForexResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncForexResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncForexResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncForexResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        code: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        foreign_per_usd: bool | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForexRetrieveResponse:
        """
        Returns forex price history for a given currency code.

        Args:
          code: Forex code

          end_date: End of forex data window (YYYY-MM-DD)

          foreign_per_usd: Return prices as foreign currency per USD. If false, prices will be returned as
              USD per foreign currency.

          start_date: Start of forex data window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not code:
            raise ValueError(f"Expected a non-empty value for `code` but received {code!r}")
        return await self._get(
            f"/v1/forex/{code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_date": end_date,
                        "foreign_per_usd": foreign_per_usd,
                        "start_date": start_date,
                    },
                    forex_retrieve_params.ForexRetrieveParams,
                ),
            ),
            cast_to=ForexRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForexListResponse:
        """Get most recent forex prices for all currencies"""
        return await self._get(
            "/v1/forex",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForexListResponse,
        )


class ForexResourceWithRawResponse:
    def __init__(self, forex: ForexResource) -> None:
        self._forex = forex

        self.retrieve = to_raw_response_wrapper(
            forex.retrieve,
        )
        self.list = to_raw_response_wrapper(
            forex.list,
        )


class AsyncForexResourceWithRawResponse:
    def __init__(self, forex: AsyncForexResource) -> None:
        self._forex = forex

        self.retrieve = async_to_raw_response_wrapper(
            forex.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            forex.list,
        )


class ForexResourceWithStreamingResponse:
    def __init__(self, forex: ForexResource) -> None:
        self._forex = forex

        self.retrieve = to_streamed_response_wrapper(
            forex.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            forex.list,
        )


class AsyncForexResourceWithStreamingResponse:
    def __init__(self, forex: AsyncForexResource) -> None:
        self._forex = forex

        self.retrieve = async_to_streamed_response_wrapper(
            forex.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            forex.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SupplyListResponse", "Data"]


class Data(BaseModel):
    commodity: str

    countries: List[str]

    model: str

    symbol: str


class SupplyListResponse(BaseModel):
    data: List[Data]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel
from .forex_data import ForexData

__all__ = ["ForexListResponse"]


class ForexListResponse(BaseModel):
    data: List[ForexData]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["FeatureCategory", "Drilldown"]


class Drilldown(BaseModel):
    contribution: float

    description: str

    feature_code: str


class FeatureCategory(BaseModel):
    category: str

    contribution: float

    drilldown: Optional[List[Drilldown]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date, datetime
from typing_extensions import Literal

from ..tick import Tick
from ..._models import BaseModel
from .futures.asset import Asset

__all__ = ["FutureGetHistoricalPricesResponse", "Data", "DataContract"]


class DataContract(BaseModel):
    asset_symbol: str

    name: str

    symbol: str

    ticks: List[Tick]

    last_tick: Optional[Tick] = None

    rollover_date: Optional[date] = None


class Data(BaseModel):
    asset: Asset

    contracts: List[DataContract]

    front_month: str

    market_date: Optional[datetime] = None


class FutureGetHistoricalPricesResponse(BaseModel):
    data: Data

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .futures.futures import (
    FuturesResource,
    AsyncFuturesResource,
    FuturesResourceWithRawResponse,
    AsyncFuturesResourceWithRawResponse,
    FuturesResourceWithStreamingResponse,
    AsyncFuturesResourceWithStreamingResponse,
)
from ...types.asset_list_response import AssetListResponse

__all__ = ["AssetsResource", "AsyncAssetsResource"]


class AssetsResource(SyncAPIResource):
    @cached_property
    def futures(self) -> FuturesResource:
        return FuturesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AssetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AssetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AssetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AssetsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetListResponse:
        """Returns a list of all available asset categories, such as futures.

        Prices,
        forecasts, and indicators are available for each.
        """
        return self._get(
            "/v1/assets",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssetListResponse,
        )


class AsyncAssetsResource(AsyncAPIResource):
    @cached_property
    def futures(self) -> AsyncFuturesResource:
        return AsyncFuturesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAssetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAssetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAssetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncAssetsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetListResponse:
        """Returns a list of all available asset categories, such as futures.

        Prices,
        forecasts, and indicators are available for each.
        """
        return await self._get(
            "/v1/assets",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssetListResponse,
        )


class AssetsResourceWithRawResponse:
    def __init__(self, assets: AssetsResource) -> None:
        self._assets = assets

        self.list = to_raw_response_wrapper(
            assets.list,
        )

    @cached_property
    def futures(self) -> FuturesResourceWithRawResponse:
        return FuturesResourceWithRawResponse(self._assets.futures)


class AsyncAssetsResourceWithRawResponse:
    def __init__(self, assets: AsyncAssetsResource) -> None:
        self._assets = assets

        self.list = async_to_raw_response_wrapper(
            assets.list,
        )

    @cached_property
    def futures(self) -> AsyncFuturesResourceWithRawResponse:
        return AsyncFuturesResourceWithRawResponse(self._assets.futures)


class AssetsResourceWithStreamingResponse:
    def __init__(self, assets: AssetsResource) -> None:
        self._assets = assets

        self.list = to_streamed_response_wrapper(
            assets.list,
        )

    @cached_property
    def futures(self) -> FuturesResourceWithStreamingResponse:
        return FuturesResourceWithStreamingResponse(self._assets.futures)


class AsyncAssetsResourceWithStreamingResponse:
    def __init__(self, assets: AsyncAssetsResource) -> None:
        self._assets = assets

        self.list = async_to_streamed_response_wrapper(
            assets.list,
        )

    @cached_property
    def futures(self) -> AsyncFuturesResourceWithStreamingResponse:
        return AsyncFuturesResourceWithStreamingResponse(self._assets.futures)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["IndicatorGetHedgeParams"]


class IndicatorGetHedgeParams(TypedDict, total=False):
    contract: Optional[str]
    """Contract year and month. _Default value_ : All available contracts"""

    end_date: Optional[str]
    """End of indicator window (YYYY-MM-DD).

    The returned object will contain every requested indicator made between
    start*date and end_date. \\__Default value* : most recent date with indicators
    """

    hedge_horizon: Optional[int]
    """Number of trading days in the future to hedge. _Default value_ : Full forecast"""

    lookback_days: int
    """
    Number of trading days to look back when computing indicator quintiles. _Default
    value_ : 30 days
    """

    start_date: Optional[str]
    """Start of indicator window (YYYY-MM-DD).

    The returned object will contain every requested indicator made between
    start*date and end_date. \\__Default value* : most recent date with indicators
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import supply_retrieve_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.supply_list_response import SupplyListResponse
from ..types.supply_retrieve_response import SupplyRetrieveResponse

__all__ = ["SupplyResource", "AsyncSupplyResource"]


class SupplyResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SupplyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return SupplyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SupplyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return SupplyResourceWithStreamingResponse(self)

    def retrieve(
        self,
        symbol: str,
        *,
        country_code: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        get_feature_contributions: bool | NotGiven = NOT_GIVEN,
        model: str | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SupplyRetrieveResponse:
        """
        Returns historical and forecasted supply data for a given commodity and country.
        Country codes follow the UN/LOCODE standard:
        https://unece.org/trade/cefact/unlocode-code-list-country-and-territory

        Args:
          symbol: Asset symbol

          country_code: Country code (UN/LOCODE). If blank, return global data.

          end_date: End of date range for supply forecasts (YYYY-MM-DD)

          get_feature_contributions: Return feature contributions for requested forecasts.

          model: Supply model to use for forecasting.

          start_date: Start of date range for supply forecasts (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return self._get(
            f"/v1/supply/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country_code": country_code,
                        "end_date": end_date,
                        "get_feature_contributions": get_feature_contributions,
                        "model": model,
                        "start_date": start_date,
                    },
                    supply_retrieve_params.SupplyRetrieveParams,
                ),
            ),
            cast_to=SupplyRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SupplyListResponse:
        """Returns a list of all commodities that have supply models available.

        Country
        codes follow the UN/LOCODE standard:
        https://unece.org/trade/cefact/unlocode-code-list-country-and-territory
        """
        return self._get(
            "/v1/supply",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SupplyListResponse,
        )


class AsyncSupplyResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSupplyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSupplyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSupplyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncSupplyResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        symbol: str,
        *,
        country_code: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        get_feature_contributions: bool | NotGiven = NOT_GIVEN,
        model: str | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SupplyRetrieveResponse:
        """
        Returns historical and forecasted supply data for a given commodity and country.
        Country codes follow the UN/LOCODE standard:
        https://unece.org/trade/cefact/unlocode-code-list-country-and-territory

        Args:
          symbol: Asset symbol

          country_code: Country code (UN/LOCODE). If blank, return global data.

          end_date: End of date range for supply forecasts (YYYY-MM-DD)

          get_feature_contributions: Return feature contributions for requested forecasts.

          model: Supply model to use for forecasting.

          start_date: Start of date range for supply forecasts (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return await self._get(
            f"/v1/supply/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country_code": country_code,
                        "end_date": end_date,
                        "get_feature_contributions": get_feature_contributions,
                        "model": model,
                        "start_date": start_date,
                    },
                    supply_retrieve_params.SupplyRetrieveParams,
                ),
            ),
            cast_to=SupplyRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SupplyListResponse:
        """Returns a list of all commodities that have supply models available.

        Country
        codes follow the UN/LOCODE standard:
        https://unece.org/trade/cefact/unlocode-code-list-country-and-territory
        """
        return await self._get(
            "/v1/supply",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SupplyListResponse,
        )


class SupplyResourceWithRawResponse:
    def __init__(self, supply: SupplyResource) -> None:
        self._supply = supply

        self.retrieve = to_raw_response_wrapper(
            supply.retrieve,
        )
        self.list = to_raw_response_wrapper(
            supply.list,
        )


class AsyncSupplyResourceWithRawResponse:
    def __init__(self, supply: AsyncSupplyResource) -> None:
        self._supply = supply

        self.retrieve = async_to_raw_response_wrapper(
            supply.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            supply.list,
        )


class SupplyResourceWithStreamingResponse:
    def __init__(self, supply: SupplyResource) -> None:
        self._supply = supply

        self.retrieve = to_streamed_response_wrapper(
            supply.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            supply.list,
        )


class AsyncSupplyResourceWithStreamingResponse:
    def __init__(self, supply: AsyncSupplyResource) -> None:
        self._supply = supply

        self.retrieve = async_to_streamed_response_wrapper(
            supply.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            supply.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["Asset"]


class Asset(BaseModel):
    asset_class_symbol: str

    code: Optional[str] = None

    name: str

    price_file: Optional[str] = None

    sku: Optional[str] = None

    symbol: str

    type: str

    exchange: Optional[str] = None

    market_date: Optional[datetime] = None

    tick_size: Optional[float] = None

    unit: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from .forecasts import (
    ForecastsResource,
    AsyncForecastsResource,
    ForecastsResourceWithRawResponse,
    AsyncForecastsResourceWithRawResponse,
    ForecastsResourceWithStreamingResponse,
    AsyncForecastsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from .indicators import (
    IndicatorsResource,
    AsyncIndicatorsResource,
    IndicatorsResourceWithRawResponse,
    AsyncIndicatorsResourceWithRawResponse,
    IndicatorsResourceWithStreamingResponse,
    AsyncIndicatorsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.assets import future_get_trading_calendar_params, future_get_historical_prices_params
from ....types.assets.future_list_response import FutureListResponse
from ....types.assets.future_get_trading_calendar_response import FutureGetTradingCalendarResponse
from ....types.assets.future_get_historical_prices_response import FutureGetHistoricalPricesResponse

__all__ = ["FuturesResource", "AsyncFuturesResource"]


class FuturesResource(SyncAPIResource):
    @cached_property
    def forecasts(self) -> ForecastsResource:
        return ForecastsResource(self._client)

    @cached_property
    def indicators(self) -> IndicatorsResource:
        return IndicatorsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FuturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return FuturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FuturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return FuturesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FutureListResponse:
        """Returns a list of all available future symbols that Hedgewise tracks."""
        return self._get(
            "/v1/assets/futures",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FutureListResponse,
        )

    def get_historical_prices(
        self,
        symbol: str,
        *,
        active_contracts_only: bool | NotGiven = NOT_GIVEN,
        back_adjust: bool | NotGiven = NOT_GIVEN,
        contract: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        rollover_method: str | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FutureGetHistoricalPricesResponse:
        """Returns historical price data for a given future symbol.

        Prices are available
        for all actively traded contracts on each day.

        Args:
          symbol: Future symbol

          active_contracts_only: Return price data for currently active contracts only. Set to false to also
              retrieve price data from expired contracts.

          back_adjust: Back-adjust prices to account for calendar spread at contract rollover dates.
              The method used is described here:
              https://www.sierrachart.com/index.php?page=doc/ContinuousFuturesContractCharts.html#ContinuousFuturesContractDateRuleRolloverBackAdjusted

          contract: Contract year and month. _Default value_ : All available contracts

          end_date: End of prices window (YYYY-MM-DD). The returned object will contain the entire
              price history for every contract that traded between `start_date` and
              `end_date`. Ignored if `contract` is specified. _Default value_ : most recent
              date with prices

          rollover_method: The rollover date is the most recent date for which a given contract was trading
              as the front month. This parameter specifies the method used to determine the
              rollover date for contracts. Must be one of "hist_vol", "max_vol",
              "first_notice", or "last_trade" (or left blank for no rollover calculation).
              "first_notice" not available for all commodities, and defaults to "last_trade".

          start_date: Start of prices window (YYYY-MM-DD). The returned object will contain the entire
              price history for every contract that traded between `start_date` and
              `end_date`. Ignored if `contract` is specified. _Default value_ : most recent
              date with prices

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return self._get(
            f"/v1/assets/futures/prices/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "active_contracts_only": active_contracts_only,
                        "back_adjust": back_adjust,
                        "contract": contract,
                        "end_date": end_date,
                        "rollover_method": rollover_method,
                        "start_date": start_date,
                    },
                    future_get_historical_prices_params.FutureGetHistoricalPricesParams,
                ),
            ),
            cast_to=FutureGetHistoricalPricesResponse,
        )

    def get_trading_calendar(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FutureGetTradingCalendarResponse:
        """
        Returns a list of all trading days for a given future symbol, based on the
        exchange where its contracts are traded. Prices, forecasts, and indicators are
        available for each of these days.

        Args:
          symbol: Future symbol

          end_date: End of trading calendar window (YYYY-MM-DD)

          start_date: Start of trading calendar window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return self._get(
            f"/v1/assets/futures/calendars/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    future_get_trading_calendar_params.FutureGetTradingCalendarParams,
                ),
            ),
            cast_to=FutureGetTradingCalendarResponse,
        )


class AsyncFuturesResource(AsyncAPIResource):
    @cached_property
    def forecasts(self) -> AsyncForecastsResource:
        return AsyncForecastsResource(self._client)

    @cached_property
    def indicators(self) -> AsyncIndicatorsResource:
        return AsyncIndicatorsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFuturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFuturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFuturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncFuturesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FutureListResponse:
        """Returns a list of all available future symbols that Hedgewise tracks."""
        return await self._get(
            "/v1/assets/futures",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FutureListResponse,
        )

    async def get_historical_prices(
        self,
        symbol: str,
        *,
        active_contracts_only: bool | NotGiven = NOT_GIVEN,
        back_adjust: bool | NotGiven = NOT_GIVEN,
        contract: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        rollover_method: str | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FutureGetHistoricalPricesResponse:
        """Returns historical price data for a given future symbol.

        Prices are available
        for all actively traded contracts on each day.

        Args:
          symbol: Future symbol

          active_contracts_only: Return price data for currently active contracts only. Set to false to also
              retrieve price data from expired contracts.

          back_adjust: Back-adjust prices to account for calendar spread at contract rollover dates.
              The method used is described here:
              https://www.sierrachart.com/index.php?page=doc/ContinuousFuturesContractCharts.html#ContinuousFuturesContractDateRuleRolloverBackAdjusted

          contract: Contract year and month. _Default value_ : All available contracts

          end_date: End of prices window (YYYY-MM-DD). The returned object will contain the entire
              price history for every contract that traded between `start_date` and
              `end_date`. Ignored if `contract` is specified. _Default value_ : most recent
              date with prices

          rollover_method: The rollover date is the most recent date for which a given contract was trading
              as the front month. This parameter specifies the method used to determine the
              rollover date for contracts. Must be one of "hist_vol", "max_vol",
              "first_notice", or "last_trade" (or left blank for no rollover calculation).
              "first_notice" not available for all commodities, and defaults to "last_trade".

          start_date: Start of prices window (YYYY-MM-DD). The returned object will contain the entire
              price history for every contract that traded between `start_date` and
              `end_date`. Ignored if `contract` is specified. _Default value_ : most recent
              date with prices

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return await self._get(
            f"/v1/assets/futures/prices/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "active_contracts_only": active_contracts_only,
                        "back_adjust": back_adjust,
                        "contract": contract,
                        "end_date": end_date,
                        "rollover_method": rollover_method,
                        "start_date": start_date,
                    },
                    future_get_historical_prices_params.FutureGetHistoricalPricesParams,
                ),
            ),
            cast_to=FutureGetHistoricalPricesResponse,
        )

    async def get_trading_calendar(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FutureGetTradingCalendarResponse:
        """
        Returns a list of all trading days for a given future symbol, based on the
        exchange where its contracts are traded. Prices, forecasts, and indicators are
        available for each of these days.

        Args:
          symbol: Future symbol

          end_date: End of trading calendar window (YYYY-MM-DD)

          start_date: Start of trading calendar window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return await self._get(
            f"/v1/assets/futures/calendars/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    future_get_trading_calendar_params.FutureGetTradingCalendarParams,
                ),
            ),
            cast_to=FutureGetTradingCalendarResponse,
        )


class FuturesResourceWithRawResponse:
    def __init__(self, futures: FuturesResource) -> None:
        self._futures = futures

        self.list = to_raw_response_wrapper(
            futures.list,
        )
        self.get_historical_prices = to_raw_response_wrapper(
            futures.get_historical_prices,
        )
        self.get_trading_calendar = to_raw_response_wrapper(
            futures.get_trading_calendar,
        )

    @cached_property
    def forecasts(self) -> ForecastsResourceWithRawResponse:
        return ForecastsResourceWithRawResponse(self._futures.forecasts)

    @cached_property
    def indicators(self) -> IndicatorsResourceWithRawResponse:
        return IndicatorsResourceWithRawResponse(self._futures.indicators)


class AsyncFuturesResourceWithRawResponse:
    def __init__(self, futures: AsyncFuturesResource) -> None:
        self._futures = futures

        self.list = async_to_raw_response_wrapper(
            futures.list,
        )
        self.get_historical_prices = async_to_raw_response_wrapper(
            futures.get_historical_prices,
        )
        self.get_trading_calendar = async_to_raw_response_wrapper(
            futures.get_trading_calendar,
        )

    @cached_property
    def forecasts(self) -> AsyncForecastsResourceWithRawResponse:
        return AsyncForecastsResourceWithRawResponse(self._futures.forecasts)

    @cached_property
    def indicators(self) -> AsyncIndicatorsResourceWithRawResponse:
        return AsyncIndicatorsResourceWithRawResponse(self._futures.indicators)


class FuturesResourceWithStreamingResponse:
    def __init__(self, futures: FuturesResource) -> None:
        self._futures = futures

        self.list = to_streamed_response_wrapper(
            futures.list,
        )
        self.get_historical_prices = to_streamed_response_wrapper(
            futures.get_historical_prices,
        )
        self.get_trading_calendar = to_streamed_response_wrapper(
            futures.get_trading_calendar,
        )

    @cached_property
    def forecasts(self) -> ForecastsResourceWithStreamingResponse:
        return ForecastsResourceWithStreamingResponse(self._futures.forecasts)

    @cached_property
    def indicators(self) -> IndicatorsResourceWithStreamingResponse:
        return IndicatorsResourceWithStreamingResponse(self._futures.indicators)


class AsyncFuturesResourceWithStreamingResponse:
    def __init__(self, futures: AsyncFuturesResource) -> None:
        self._futures = futures

        self.list = async_to_streamed_response_wrapper(
            futures.list,
        )
        self.get_historical_prices = async_to_streamed_response_wrapper(
            futures.get_historical_prices,
        )
        self.get_trading_calendar = async_to_streamed_response_wrapper(
            futures.get_trading_calendar,
        )

    @cached_property
    def forecasts(self) -> AsyncForecastsResourceWithStreamingResponse:
        return AsyncForecastsResourceWithStreamingResponse(self._futures.forecasts)

    @cached_property
    def indicators(self) -> AsyncIndicatorsResourceWithStreamingResponse:
        return AsyncIndicatorsResourceWithStreamingResponse(self._futures.indicators)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hedgewise import Hedgewise, AsyncHedgewise
from tests.utils import assert_matches_type
from hedgewise.types.assets.futures import IndicatorListResponse, IndicatorGetHedgeResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestIndicators:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hedgewise) -> None:
        indicator = client.assets.futures.indicators.list()
        assert_matches_type(IndicatorListResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hedgewise) -> None:
        response = client.assets.futures.indicators.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        indicator = response.parse()
        assert_matches_type(IndicatorListResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hedgewise) -> None:
        with client.assets.futures.indicators.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            indicator = response.parse()
            assert_matches_type(IndicatorListResponse, indicator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_hedge(self, client: Hedgewise) -> None:
        indicator = client.assets.futures.indicators.get_hedge(
            symbol="ZC",
        )
        assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_hedge_with_all_params(self, client: Hedgewise) -> None:
        indicator = client.assets.futures.indicators.get_hedge(
            symbol="ZC",
            contract="2025H",
            end_date="2025-04-24",
            hedge_horizon=5,
            lookback_days=30,
            start_date="2025-03-24",
        )
        assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_hedge(self, client: Hedgewise) -> None:
        response = client.assets.futures.indicators.with_raw_response.get_hedge(
            symbol="ZC",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        indicator = response.parse()
        assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_hedge(self, client: Hedgewise) -> None:
        with client.assets.futures.indicators.with_streaming_response.get_hedge(
            symbol="ZC",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            indicator = response.parse()
            assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_hedge(self, client: Hedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `symbol` but received ''"):
            client.assets.futures.indicators.with_raw_response.get_hedge(
                symbol="",
            )


class TestAsyncIndicators:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHedgewise) -> None:
        indicator = await async_client.assets.futures.indicators.list()
        assert_matches_type(IndicatorListResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.assets.futures.indicators.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        indicator = await response.parse()
        assert_matches_type(IndicatorListResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHedgewise) -> None:
        async with async_client.assets.futures.indicators.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            indicator = await response.parse()
            assert_matches_type(IndicatorListResponse, indicator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_hedge(self, async_client: AsyncHedgewise) -> None:
        indicator = await async_client.assets.futures.indicators.get_hedge(
            symbol="ZC",
        )
        assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_hedge_with_all_params(self, async_client: AsyncHedgewise) -> None:
        indicator = await async_client.assets.futures.indicators.get_hedge(
            symbol="ZC",
            contract="2025H",
            end_date="2025-04-24",
            hedge_horizon=5,
            lookback_days=30,
            start_date="2025-03-24",
        )
        assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_hedge(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.assets.futures.indicators.with_raw_response.get_hedge(
            symbol="ZC",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        indicator = await response.parse()
        assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_hedge(self, async_client: AsyncHedgewise) -> None:
        async with async_client.assets.futures.indicators.with_streaming_response.get_hedge(
            symbol="ZC",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            indicator = await response.parse()
            assert_matches_type(IndicatorGetHedgeResponse, indicator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_hedge(self, async_client: AsyncHedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `symbol` but received ''"):
            await async_client.assets.futures.indicators.with_raw_response.get_hedge(
                symbol="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .futures import (
    FuturesResource,
    AsyncFuturesResource,
    FuturesResourceWithRawResponse,
    AsyncFuturesResourceWithRawResponse,
    FuturesResourceWithStreamingResponse,
    AsyncFuturesResourceWithStreamingResponse,
)
from .forecasts import (
    ForecastsResource,
    AsyncForecastsResource,
    ForecastsResourceWithRawResponse,
    AsyncForecastsResourceWithRawResponse,
    ForecastsResourceWithStreamingResponse,
    AsyncForecastsResourceWithStreamingResponse,
)
from .indicators import (
    IndicatorsResource,
    AsyncIndicatorsResource,
    IndicatorsResourceWithRawResponse,
    AsyncIndicatorsResourceWithRawResponse,
    IndicatorsResourceWithStreamingResponse,
    AsyncIndicatorsResourceWithStreamingResponse,
)

__all__ = [
    "ForecastsResource",
    "AsyncForecastsResource",
    "ForecastsResourceWithRawResponse",
    "AsyncForecastsResourceWithRawResponse",
    "ForecastsResourceWithStreamingResponse",
    "AsyncForecastsResourceWithStreamingResponse",
    "IndicatorsResource",
    "AsyncIndicatorsResource",
    "IndicatorsResourceWithRawResponse",
    "AsyncIndicatorsResourceWithRawResponse",
    "IndicatorsResourceWithStreamingResponse",
    "AsyncIndicatorsResourceWithStreamingResponse",
    "FuturesResource",
    "AsyncFuturesResource",
    "FuturesResourceWithRawResponse",
    "AsyncFuturesResourceWithRawResponse",
    "FuturesResourceWithStreamingResponse",
    "AsyncFuturesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PerformanceRetrieveResponse", "Data"]


class Data(BaseModel):
    horizon: int

    metric: str

    value: float

    date: Optional[str] = None


class PerformanceRetrieveResponse(BaseModel):
    data: List[Data]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["FeatureListParams"]


class FeatureListParams(TypedDict, total=False):
    dataset_key: Optional[str]
    """Dataset key to which the features"""

    symbol: Optional[str]
    """Futures contract symbol"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TransformedFeature", "Data"]


class Data(BaseModel):
    date: datetime.date

    value: float

    xform_feature_code: str

    label: Optional[str] = None


class TransformedFeature(BaseModel):
    data: List[Data]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from .asset import Asset
from ...tick import Tick
from ...._models import BaseModel
from ...feature_category import FeatureCategory

__all__ = [
    "ForecastGetResponse",
    "GetAssetForecastsResponse",
    "GetAssetForecastsResponseData",
    "GetAssetForecastsResponseDataContract",
    "GetAssetForecastsResponseDataContractForecast",
    "GetAssetForecastsResponseDataContractForecastClosePriceTrajectory",
    "GetAssetForecastsResponseDataContractMarketDriver",
    "GetAssetForecastsResponseDataContractMovingAverage",
    "GetAssetForecastsResponseDataContractMovingAverageMovingAverage",
    "GetAssetForecastsSmallResponse",
    "GetAssetForecastsSmallResponseData",
    "GetAssetForecastsSmallResponseDataContract",
    "GetAssetForecastsSmallResponseDataContractForecast",
    "GetAssetForecastsSmallResponseDataContractForecastClosePriceTrajectory",
    "GetAssetForecastsSmallResponseDataContractMarketDriver",
    "GetAssetForecastsSmallResponseDataContractMovingAverage",
    "GetAssetForecastsSmallResponseDataContractMovingAverageMovingAverage",
]


class GetAssetForecastsResponseDataContractForecastClosePriceTrajectory(BaseModel):
    close_price: float

    date: datetime

    target_date_contract: str

    interpolated: Optional[bool] = None

    lower_bound_1_sigma: Optional[float] = None

    lower_bound_2_sigma: Optional[float] = None

    lower_bound_3_sigma: Optional[float] = None

    upper_bound_1_sigma: Optional[float] = None

    upper_bound_2_sigma: Optional[float] = None

    upper_bound_3_sigma: Optional[float] = None


class GetAssetForecastsResponseDataContractForecast(BaseModel):
    close_price_trajectory: List[GetAssetForecastsResponseDataContractForecastClosePriceTrajectory]

    forecast_date: datetime

    model: str


class GetAssetForecastsResponseDataContractMarketDriver(BaseModel):
    categories: List[FeatureCategory]

    forecast_date: datetime

    horizon: int

    model: str

    target_date_contract: str


class GetAssetForecastsResponseDataContractMovingAverageMovingAverage(BaseModel):
    date: datetime

    value: float


class GetAssetForecastsResponseDataContractMovingAverage(BaseModel):
    horizon: int

    moving_average: List[GetAssetForecastsResponseDataContractMovingAverageMovingAverage]


class GetAssetForecastsResponseDataContract(BaseModel):
    asset_symbol: str

    forecasts: List[GetAssetForecastsResponseDataContractForecast]

    market_drivers: Optional[List[GetAssetForecastsResponseDataContractMarketDriver]] = None

    moving_averages: Optional[List[GetAssetForecastsResponseDataContractMovingAverage]] = None

    name: str

    symbol: str

    last_tick: Optional[Tick] = None


class GetAssetForecastsResponseData(BaseModel):
    asset: Asset

    contracts: List[GetAssetForecastsResponseDataContract]


class GetAssetForecastsResponse(BaseModel):
    data: GetAssetForecastsResponseData

    success: Literal[True]


class GetAssetForecastsSmallResponseDataContractForecastClosePriceTrajectory(BaseModel):
    close_price: float

    date: datetime

    target_date_contract: str

    interpolated: Optional[bool] = None


class GetAssetForecastsSmallResponseDataContractForecast(BaseModel):
    close_price_trajectory: List[GetAssetForecastsSmallResponseDataContractForecastClosePriceTrajectory]

    forecast_date: datetime

    model: str


class GetAssetForecastsSmallResponseDataContractMarketDriver(BaseModel):
    categories: List[FeatureCategory]

    forecast_date: datetime

    horizon: int

    model: str

    target_date_contract: str


class GetAssetForecastsSmallResponseDataContractMovingAverageMovingAverage(BaseModel):
    date: datetime

    value: float


class GetAssetForecastsSmallResponseDataContractMovingAverage(BaseModel):
    horizon: int

    moving_average: List[GetAssetForecastsSmallResponseDataContractMovingAverageMovingAverage]


class GetAssetForecastsSmallResponseDataContract(BaseModel):
    asset_symbol: str

    forecasts: List[GetAssetForecastsSmallResponseDataContractForecast]

    market_drivers: Optional[List[GetAssetForecastsSmallResponseDataContractMarketDriver]] = None

    moving_averages: Optional[List[GetAssetForecastsSmallResponseDataContractMovingAverage]] = None

    name: str

    symbol: str

    last_tick: Optional[Tick] = None


class GetAssetForecastsSmallResponseData(BaseModel):
    asset: Asset

    contracts: List[GetAssetForecastsSmallResponseDataContract]


class GetAssetForecastsSmallResponse(BaseModel):
    data: GetAssetForecastsSmallResponseData

    success: Literal[True]


ForecastGetResponse: TypeAlias = Union[GetAssetForecastsResponse, GetAssetForecastsSmallResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["FeatureGetWeightedIndexParams"]


class FeatureGetWeightedIndexParams(TypedDict, total=False):
    feature_codes: Required[List[str]]
    """The list of features to include in the index"""

    index_label: Required[str]
    """user defined string used to name the weighted index"""

    weights: Required[Iterable[float]]
    """The list of weights to apply on the features selection to create the index"""

    end_date: Optional[str]
    """End of transformed feature data window (YYYY-MM-DD)"""

    start_date: Optional[str]
    """Start of transformed feature data window (YYYY-MM-DD)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.data_retrieve_response import DataRetrieveResponse

__all__ = ["DataResource", "AsyncDataResource"]


class DataResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return DataResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DataRetrieveResponse:
        """
        Returns a list of all asset classes, assets, and currently active contracts
        tracked by Hedgewise, as well as their prices from the prior trading day. Used
        to build the table on the Hedgewise landing page.
        """
        return self._get(
            "/v1/data",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataRetrieveResponse,
        )


class AsyncDataResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncDataResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DataRetrieveResponse:
        """
        Returns a list of all asset classes, assets, and currently active contracts
        tracked by Hedgewise, as well as their prices from the prior trading day. Used
        to build the table on the Hedgewise landing page.
        """
        return await self._get(
            "/v1/data",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataRetrieveResponse,
        )


class DataResourceWithRawResponse:
    def __init__(self, data: DataResource) -> None:
        self._data = data

        self.retrieve = to_raw_response_wrapper(
            data.retrieve,
        )


class AsyncDataResourceWithRawResponse:
    def __init__(self, data: AsyncDataResource) -> None:
        self._data = data

        self.retrieve = async_to_raw_response_wrapper(
            data.retrieve,
        )


class DataResourceWithStreamingResponse:
    def __init__(self, data: DataResource) -> None:
        self._data = data

        self.retrieve = to_streamed_response_wrapper(
            data.retrieve,
        )


class AsyncDataResourceWithStreamingResponse:
    def __init__(self, data: AsyncDataResource) -> None:
        self._data = data

        self.retrieve = async_to_streamed_response_wrapper(
            data.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime as _datetime
from typing import Optional

from .._models import BaseModel

__all__ = ["Tick"]


class Tick(BaseModel):
    datetime: _datetime.datetime

    open_interest: int

    volume: int

    change: Optional[float] = None

    close: Optional[float] = None

    high: Optional[float] = None

    low: Optional[float] = None

    open: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.assets.futures import indicator_get_hedge_params
from ....types.assets.futures.indicator_list_response import IndicatorListResponse
from ....types.assets.futures.indicator_get_hedge_response import IndicatorGetHedgeResponse

__all__ = ["IndicatorsResource", "AsyncIndicatorsResource"]


class IndicatorsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IndicatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return IndicatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IndicatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return IndicatorsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IndicatorListResponse:
        """
        Returns the list of all available market indicators that Hedgewise tracks or
        produces.
        """
        return self._get(
            "/v1/assets/futures/indicators",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IndicatorListResponse,
        )

    def get_hedge(
        self,
        symbol: str,
        *,
        contract: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        hedge_horizon: Optional[int] | NotGiven = NOT_GIVEN,
        lookback_days: int | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IndicatorGetHedgeResponse:
        """
        Returns a list of values indicating the likelihood that the future price will
        increase, indicating that this is a good time to hedge. The indicator is
        quintiles of the ratio of the forecasted price trajectory (out to a specified
        horizon) to the forecasted ulcer index, where the quintiles are given by
        historical values of this indicator back a specified number of lookback days.

        Args:
          symbol: Future symbol

          contract: Contract year and month. _Default value_ : All available contracts

          end_date: End of indicator window (YYYY-MM-DD). The returned object will contain every
              requested indicator made between start*date and end_date. \\__Default value* :
              most recent date with indicators

          hedge_horizon: Number of trading days in the future to hedge. _Default value_ : Full forecast

          lookback_days: Number of trading days to look back when computing indicator quintiles. _Default
              value_ : 30 days

          start_date: Start of indicator window (YYYY-MM-DD). The returned object will contain every
              requested indicator made between start*date and end_date. \\__Default value* :
              most recent date with indicators

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return self._get(
            f"/v1/assets/futures/indicators/hedge/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "contract": contract,
                        "end_date": end_date,
                        "hedge_horizon": hedge_horizon,
                        "lookback_days": lookback_days,
                        "start_date": start_date,
                    },
                    indicator_get_hedge_params.IndicatorGetHedgeParams,
                ),
            ),
            cast_to=IndicatorGetHedgeResponse,
        )


class AsyncIndicatorsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIndicatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIndicatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIndicatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncIndicatorsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IndicatorListResponse:
        """
        Returns the list of all available market indicators that Hedgewise tracks or
        produces.
        """
        return await self._get(
            "/v1/assets/futures/indicators",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IndicatorListResponse,
        )

    async def get_hedge(
        self,
        symbol: str,
        *,
        contract: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        hedge_horizon: Optional[int] | NotGiven = NOT_GIVEN,
        lookback_days: int | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IndicatorGetHedgeResponse:
        """
        Returns a list of values indicating the likelihood that the future price will
        increase, indicating that this is a good time to hedge. The indicator is
        quintiles of the ratio of the forecasted price trajectory (out to a specified
        horizon) to the forecasted ulcer index, where the quintiles are given by
        historical values of this indicator back a specified number of lookback days.

        Args:
          symbol: Future symbol

          contract: Contract year and month. _Default value_ : All available contracts

          end_date: End of indicator window (YYYY-MM-DD). The returned object will contain every
              requested indicator made between start*date and end_date. \\__Default value* :
              most recent date with indicators

          hedge_horizon: Number of trading days in the future to hedge. _Default value_ : Full forecast

          lookback_days: Number of trading days to look back when computing indicator quintiles. _Default
              value_ : 30 days

          start_date: Start of indicator window (YYYY-MM-DD). The returned object will contain every
              requested indicator made between start*date and end_date. \\__Default value* :
              most recent date with indicators

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return await self._get(
            f"/v1/assets/futures/indicators/hedge/{symbol}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "contract": contract,
                        "end_date": end_date,
                        "hedge_horizon": hedge_horizon,
                        "lookback_days": lookback_days,
                        "start_date": start_date,
                    },
                    indicator_get_hedge_params.IndicatorGetHedgeParams,
                ),
            ),
            cast_to=IndicatorGetHedgeResponse,
        )


class IndicatorsResourceWithRawResponse:
    def __init__(self, indicators: IndicatorsResource) -> None:
        self._indicators = indicators

        self.list = to_raw_response_wrapper(
            indicators.list,
        )
        self.get_hedge = to_raw_response_wrapper(
            indicators.get_hedge,
        )


class AsyncIndicatorsResourceWithRawResponse:
    def __init__(self, indicators: AsyncIndicatorsResource) -> None:
        self._indicators = indicators

        self.list = async_to_raw_response_wrapper(
            indicators.list,
        )
        self.get_hedge = async_to_raw_response_wrapper(
            indicators.get_hedge,
        )


class IndicatorsResourceWithStreamingResponse:
    def __init__(self, indicators: IndicatorsResource) -> None:
        self._indicators = indicators

        self.list = to_streamed_response_wrapper(
            indicators.list,
        )
        self.get_hedge = to_streamed_response_wrapper(
            indicators.get_hedge,
        )


class AsyncIndicatorsResourceWithStreamingResponse:
    def __init__(self, indicators: AsyncIndicatorsResource) -> None:
        self._indicators = indicators

        self.list = async_to_streamed_response_wrapper(
            indicators.list,
        )
        self.get_hedge = async_to_streamed_response_wrapper(
            indicators.get_hedge,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PerformanceListResponse", "Data"]


class Data(BaseModel):
    description: str

    metric: str


class PerformanceListResponse(BaseModel):
    data: List[Data]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .tick import Tick
from .._models import BaseModel
from .assets.futures.asset import Asset

__all__ = ["DataRetrieveResponse", "Data", "DataAssetClass", "DataContract"]


class DataAssetClass(BaseModel):
    name: str

    symbol: str


class DataContract(BaseModel):
    asset_symbol: str

    name: str

    symbol: str

    last_tick: Optional[Tick] = None


class Data(BaseModel):
    asset_classes: List[DataAssetClass]

    assets: List[Asset]

    contracts: List[DataContract]


class DataRetrieveResponse(BaseModel):
    data: Data

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["SupplyRetrieveParams"]


class SupplyRetrieveParams(TypedDict, total=False):
    country_code: Optional[str]
    """Country code (UN/LOCODE). If blank, return global data."""

    end_date: Optional[str]
    """End of date range for supply forecasts (YYYY-MM-DD)"""

    get_feature_contributions: bool
    """Return feature contributions for requested forecasts."""

    model: str
    """Supply model to use for forecasting."""

    start_date: Optional[str]
    """Start of date range for supply forecasts (YYYY-MM-DD)"""
